/*
 * A file accessor using the filesystem client interface.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <stdio.h>
#include <string.h>

#include <algorithm>

#include <fsclient/client.h>
#include <systypes/fcntl.h>

#include "client_file_accessor.h"



ClientFileAccessor::ClientFileAccessor(const char *path, fileid_t fileid)
: Accessor(fileid), _path(path)
{
}

/* Return the size of the file. */

offset_t ClientFileAccessor::get_size()
{
    return _file->size;
}

/* Update the size of the file. */

void ClientFileAccessor::set_size(offset_t size)
{
    file_resize(_file, size);
}

/* Perform any closing operation on the file. */

void ClientFileAccessor::close()
{   
    client_close(_file);
}

/* Perform any opening operation on the file. */

void ClientFileAccessor::open()
{
    _file = client_open(_path, O_RDWR);
}

/* Data transfer helper methods. */

/* Fill the populated portion of a flexpage. */

void ClientFileAccessor::fill_populated(Flexpage *flexpage)
{
    offset_t filepos = flexpage->base_offset;
    offset_t addr = flexpage->base_addr;

    /* Tag the region with file state. */

    flexpage->region->fill(fileid, filepos);

    /* Fill the region with file content. */

    client_seek(_file, filepos, SEEK_SET);
    offset_t nread = client_read(_file, (char *) addr, flexpage->size);

    /* Pad the flexpage with zero. */

    memset((void *) (addr + nread), 0, flexpage->size - nread);
}

/* Flush the populated portion of a flexpage. */

void ClientFileAccessor::flush_populated(Flexpage *flexpage)
{
    offset_t filepos = flexpage->base_offset;
    offset_t addr = flexpage->base_addr;
    offset_t populated_size = std::min(flexpage->size, get_size() - filepos);

    /* Remove the file state tag from the region. */

    flexpage->region->flush();

    /* Copy the populated region to the file. */

    client_seek(_file, filepos, SEEK_SET);
    client_write(_file, (char *) addr, populated_size);
}

// vim: tabstop=4 expandtab shiftwidth=4
